@@ -1,6 +1,6 @@
 package EBook::Ishmael::EBook::PalmDoc;
 use 5.016;
-our $VERSION = '0.07';
+our $VERSION = '1.00';
 use strict;
 use warnings;
 
@@ -18,13 +18,10 @@ sub heuristic {
 
 	my $class = shift;
 	my $file  = shift;
+	my $fh    = shift;
 
 	return 0 unless -s $file >= 68;
 
-	open my $fh, '<', $file
-		or die "Failed to to open $file for reading: $!\n";
-	binmode $fh;
-
 	seek $fh, 32, 0;
 	read $fh, my ($null), 1;
 
@@ -37,8 +34,6 @@ sub heuristic {
 	read $fh, my ($type),    4;
 	read $fh, my ($creator), 4;
 
-	close $fh;
-
 	return $type eq $TYPE && $creator eq $CREATOR;
 
 }
@@ -102,8 +97,13 @@ sub new {
 	}
 
 	$self->{Metadata}->title([ $self->{_pdb}->name ]);
-	$self->{Metadata}->created([ scalar gmtime $self->{_pdb}->cdate ]);
-	$self->{Metadata}->modified([ scalar gmtime $self->{_pdb}->mdate ]);
+
+	if ($self->{_pdb}->cdate) {
+		$self->{Metadata}->created([ scalar gmtime $self->{_pdb}->cdate ]);
+	}
+	if ($self->{_pdb}->mdate) {
+		$self->{Metadata}->modified([ scalar gmtime $self->{_pdb}->mdate ]);
+	}
 
 	if ($self->{_pdb}->version) {
 		$self->{Metadata}->format([
@@ -171,4 +171,8 @@ sub has_cover { 0 }
 
 sub cover { undef }
 
+sub image_num { 0 }
+
+sub image { undef }
+
 1;
